@@ -15,10 +15,7 @@ use constant DEBUG => $ENV{MOJO_EVENTEMITTER_DEBUG} || 0;
 sub emit      { shift->_emit(0, @_) }
 sub emit_safe { shift->_emit(1, @_) }
 
-sub has_subscribers {
-  return 1 if @{shift->subscribers(shift)};
-  return;
-}
+sub has_subscribers { scalar @{shift->subscribers(shift)} }
 
 sub on {
   my ($self, $name, $cb) = @_;
@@ -120,6 +117,7 @@ Mojo::EventEmitter - Event emitter base class
 =head1 DESCRIPTION
 
 L<Mojo::EventEmitter> is a simple base class for event emitting objects.
+
 Note that this module is EXPERIMENTAL and might change without warning!
 
 =head1 METHODS
