@@ -121,7 +121,7 @@ sub connection   { shift->handshake->connection(@_) }
 
 sub finish {
   my $self = shift;
-  $self->_send_frame(CLOSE, '');
+  $self->send_frame(1, CLOSE, '');
   $self->{finished} = 1;
   return $self;
 }
@@ -216,17 +216,30 @@ sub resume {
   return $self;
 }
 
-sub send_message {
-  my ($self, $message, $cb) = @_;
-  $message //= '';
+sub send_frame {
+  my ($self, $fin, $type, $payload, $cb) = @_;
 
-  # Binary
+  # Prepare frame
   $self->{drain} = $cb if $cb;
-  return $self->_send_frame(BINARY, $message->[0]) if ref $message;
+  $self->{write} //= '';
+  $self->{write} .= $self->build_frame($fin, $type, $payload);
+  $self->{state} = 'write';
+
+  # Resume
+  $self->emit('resume');
+}
+
+sub send_message {
+  my ($self, $m, $cb) = @_;
+  $m //= '';
+
+  # Binary or raw text
+  return $self->send_frame(1, $m->[0] eq 'text' ? TEXT : BINARY, $m->[1], $cb)
+    if ref $m;
 
   # Text
-  encode 'UTF-8', $message;
-  $self->_send_frame(TEXT, $message);
+  encode 'UTF-8', $m;
+  $self->send_frame(1, TEXT, $m, $cb);
 }
 
 sub server_handshake {
@@ -256,21 +269,18 @@ sub server_read {
   $self->{read} .= $chunk if defined $chunk;
   $self->{message} //= '';
   while (my $frame = $self->parse_frame(\$self->{read})) {
+    $self->emit(frame => $frame);
     my $op = $frame->[1] || CONTINUATION;
 
-    # Ping
-    if ($op == PING) {
-      $self->_send_frame(PONG, $frame->[2]);
-      next;
-    }
+    # Ping/Pong
+    $self->send_frame(1, PONG, $frame->[2]) and next if $op == PING;
+    next if $op == PONG;
 
     # Close
-    elsif ($op == CLOSE) {
-      $self->finish;
-      next;
-    }
+    $self->finish and next if $op == CLOSE;
 
     # Append chunk and check message size
+    next unless $self->has_subscribers('message');
     $self->{op} = $op unless exists $self->{op};
     $self->{message} .= $frame->[2];
     $self->finish and last
@@ -283,7 +293,6 @@ sub server_read {
     my $message = $self->{message};
     $self->{message} = '';
     decode 'UTF-8', $message if $message && delete $self->{op} == TEXT;
-    return $self->finish unless $self->has_subscribers('message');
     $self->emit(message => $message);
   }
 
@@ -324,16 +333,6 @@ sub _challenge {
   return $challenge;
 }
 
-sub _send_frame {
-  my ($self, $op, $payload) = @_;
-
-  # Build frame and resume
-  $self->{write} //= '';
-  $self->{write} .= $self->build_frame(1, $op, $payload);
-  $self->{state} = 'write';
-  $self->emit('resume');
-}
-
 sub _xor_mask {
   my ($input, $mask) = @_;
 
@@ -357,11 +356,14 @@ Mojo::Transaction::WebSocket - WebSocket transaction container
 
   use Mojo::Transaction::WebSocket;
 
+  my $ws = Mojo::Transaction::WebSocket->new;
+
 =head1 DESCRIPTION
 
 L<Mojo::Transaction::WebSocket> is a container for WebSocket transactions as
 described in
 L<http://www.ietf.org/id/draft-ietf-hybi-thewebsocketprotocol-17.txt>.
+
 Note that this module is EXPERIMENTAL and might change without warning!
 
 =head1 EVENTS
@@ -369,13 +371,28 @@ Note that this module is EXPERIMENTAL and might change without warning!
 L<Mojo::Transaction::WebSocket> inherits all events from L<Mojo::Transaction>
 and can emit the following new ones.
 
+=head2 C<frame>
+
+  $ws->on(frame => sub {
+    my ($ws, $frame) = @_;
+  });
+
+Emitted when a WebSocket frame has been received.
+
+  $ws->on(frame => sub {
+    my ($ws, $frame) = @_;
+    say 'Fin: ',     $frame->[0];
+    say 'Type: ',    $frame->[1];
+    say 'Payload: ', $frame->[2];
+  });
+
 =head2 C<message>
 
   $ws->on(message => sub {
     my ($ws, $message) = @_;
   });
 
-Emitted when a new WebSocket message arrives.
+Emitted when a complete WebSocket message has been received.
 
   $ws->on(message => sub {
     my ($ws, $message) = @_;
@@ -511,12 +528,20 @@ The original handshake response.
 
 Resume transaction.
 
+=head2 C<send_frame>
+
+  $ws->send_frame($fin, $op, $payload);
+  $ws->send_frame($fin, $op, $payload, sub {...});
+
+Send a single frame non-blocking via WebSocket, the optional drain callback
+will be invoked once all data has been written.
+
 =head2 C<send_message>
 
+  $ws->send_message([binary => $bytes]);
+  $ws->send_message([text   => $bytes]);
   $ws->send_message('Hi there!');
   $ws->send_message('Hi there!', sub {...});
-  $ws->send_message([$bytes]);
-  $ws->send_message([$bytes], sub {...});
 
 Send a message non-blocking via WebSocket, the optional drain callback will
 be invoked once all data has been written.
