@@ -55,7 +55,7 @@ sub run {
     elsif ($code eq '409') { $message = 'File already exists on CPAN.' }
     die qq/Problem uploading file "$file". ($message)\n/;
   }
-  print "Upload sucessful!\n";
+  say 'Upload sucessful!';
 }
 
 1;
@@ -75,6 +75,7 @@ Mojolicious::Command::cpanify - Cpanify command
 =head1 DESCRIPTION
 
 L<Mojolicious::Command::cpanify> is a CPAN uploader.
+
 Note that this module is EXPERIMENTAL and might change without warning!
 
 =head1 ATTRIBUTES
